//https://calendee.com/2013/12/29/angularjs-code-organization/
angular.module('starter.controllers')
//WELCOME PAGE
.controller('WelcomeController', function($scope, $state, $rootScope, $ionicLoading, $ionicHistory, $stateParams, $cordovaGeolocation, $ionicPlatform, ClockSrv) {
    console.log("WelcomeController");
    ClockSrv.stopClock();

    $scope.user = {
        username: null,
        password: null
    };

    $scope.error = {};

    var posOptions = null;

    // If android
    if($ionicPlatform.is('android')){
      console.log("Updating settings for Android (Geolocation)");
      posOptions = {timeout: 3600000, maximumAge:10000, enableHighAccuracy: true};
    }
    // If iPhone
    else {
      console.log("Updating settings for iOS (Geolocation)");
      posOptions = {timeout: 4000, maximumAge: 3000 , enableHighAccuracy: false};
    }

    var currentPositionID = $cordovaGeolocation.getCurrentPosition(posOptions);
        currentPositionID.then(
        null,
        function(err) {
            console.log("Error watching geoLocation, err=" + JSON.stringify(err));
        },
        function(position) {
            $rootScope.currentLiveLatitude  = position.coords.latitude;
            $rootScope.currentLiveLongitude = position.coords.longitude;
    });

    try {
      if (window.localStorage.getItem("cachedUserName")){
        $ionicLoading.show({
              duration: 6000,
              noBackdrop: false,
              template: '<h2>Logging In</h2>'
        });

        console.log("Cached user found locally!");

        Parse.initialize('ZmqvFWg6V7u0ny8bX24u7PAnKvfewOvRDxfNdoxH', 'HlzTePUSlhYKCukte0Jkx0BoITzuxezBs7Zvo521');
        Parse.User.logIn((window.localStorage.getItem("cachedUserName")) , window.localStorage.getItem("cachedUP"), {
            success: function(user) {
                console.log('Login w/ Cached credentials successful');
                console.log('user = ' + user.attributes.username);
                console.log('email = ' + user.attributes.email);
              
                $ionicLoading.hide();
                $rootScope.user = Parse.User.current();
                $state.go('tab.instructions');
            },
            error: function(user, err) {
                console.log("Logging in w/ cached credentials error");
                $ionicLoading.hide();
                // The login failed. Check error to see why.
                if (err.code === 101) {
                    $scope.error.message = 'Invalid login credentials';
                } else {
                    $scope.error.message = 'An unexpected error has ' +
                        'occurred, please try again.';
                }
                $scope.$apply();
            }
        });
      }
      else {
        console.log("No existing user found");
        $state.go('welcome');
      }
    }
    catch(err) {
      console.log("No local cache was found for user " + err);
    }

	//LOGIN LINK
	$scope.signInClick = function() {
    	console.log('Login was clicked');
        console.log("Username = " + $scope.user.username);
        console.log("password = " + $scope.user.password);

        $scope.loading = $ionicLoading.show({
            content: 'Logging in',
            animation: 'fade-in',
            showBackdrop: true,
            maxWidth: 200,
            showDelay: 0
        });

        Parse.User.logIn(($scope.user.username) , $scope.user.password, {
            success: function(user) {
                console.log('Login Success');
                console.log('user = ' + user.attributes.username);
                console.log('email = ' + user.attributes.email);
                window.localStorage.setItem("cachedUserName", $scope.user.username);
                window.localStorage.setItem("cachedUP", $scope.user.password);

                $ionicLoading.hide();
                $rootScope.user = Parse.User.current();
                $state.go('tab.instructions');
            },
            error: function(user, err) {
                console.log("Inside Parse.User.logIn error");
                $ionicLoading.hide();
                // The login failed. Check error to see why.
                if (err.code === 101) {
                    $scope.error.message = 'Invalid login credentials';
                } else {
                    $scope.error.message = 'An unexpected error has ' +
                        'occurred, please try again.';
                }
                $scope.$apply();
            }
        });
	};

    // FORGOT PASSWORD LINK
    $scope.forgot = function() {
        console.log('Login Controller --> Forgot Password was clicked :)');
        $state.go('forgot');
    };

	//LINK TO FACEBOOK LOGIN
	//linking with Facebook: https://www.parse.com/docs/js/symbols/Parse.FacebookUtils.html
	$scope.loginWithFacebookClick = function () {
		console.log('Login with Facebook was clicked');
	};

	//LINK TO SIGNUP PAGE
	$scope.signUpClick = function() {
	    console.log('Sign up was clicked');
	    $state.go('register');
	};

})

.controller('ProfileCtrl', function($scope, $state, $rootScope, $stateParams, $ionicLoading, $ionicPopup) {
    console.log('ProfileController');

    var currentUser = Parse.User.current();

    console.log("Username = " + currentUser.attributes.username);
    console.log("email = " + currentUser.attributes.email);
    console.log("Industry = " + currentUser.attributes.industry);
    console.log("manufacturer = " + currentUser.attributes.manufacturer);

    try {
        $scope.user.email = currentUser.attributes.email;
        $scope.user.industry = currentUser.attributes.industry;
        $scope.user.manufacturer = currentUser.attributes.manufacturer;
    }
    catch (err) {
        console.log("Error setting the scope.email and attributes, err = " + err);
        var alertPopup = $ionicPopup.alert({
             title: 'Could not retrieve profile, please restart the app!'
        });
        alertPopup.then(function(res) {
            console.log('User clicked continue for Geolocation popup');
        });
    }

    $scope.error = {};

    $scope.updateProfileClick = function () {
        console.log('Update Profile was clicked');
        $ionicLoading.show({
            duration: 3000,
            noBackdrop: false,
            template: '<h2>Updating Profile</h2>'
        });

        currentUser.save(null, {
            success: function(updatedUser) {
                console.log("Setting email to : " + $scope.user.email);
                console.log('industry =' + $scope.user.industry);
                console.log('manufacturer =' + $scope.user.manufacturer);
                
                updatedUser.set('email', $scope.user.email);
                updatedUser.set("industry", $scope.user.industry);
                updatedUser.set("manufacturer", $scope.user.manufacturer);
                
                updatedUser.save();
            }
        });
    };

    $scope.profileLogout = function() {
        console.log("Logging out from ProfileCtrl");
        
        currentUser = null;
        Parse.User.logOut();
        
        window.localStorage.clear();
        console.log("Log out was called to Parse");

        $rootScope.user = null;
        $state.go('welcome');
    };
})

.controller('RegisterController', function($scope, $state, $ionicLoading, $rootScope, $ionicPopup) {
    console.log('inside the RegisterController');
    $scope.user = {};
    $scope.error = {};

    $scope.registerNowClick = function () {
        
        try {
            console.log('Register was clicked');

            var user = new Parse.User();
            user.set("username", $scope.user.username);
            user.set("password", $scope.user.password);
            user.set("email", $scope.user.email);

            console.log("Username = " + $scope.user.username);
            console.log("email = " + $scope.user.email);
            console.log("password = " + $scope.user.password);
            
            if ($scope.user.passwordConfirm === undefined){
                $scope.error.message = 'Please confirm your password.';
            }
            else if ($scope.user.password === $scope.user.passwordConfirm)
            {
               var confirmPopup = $ionicPopup.confirm({
                    title: 'Terms and Conditions',
                    template: '<div class="padding-vertical sides" style="text-align:justify;">By clicking OK you agree to use TRUCKBLOX as a travel aid only, not rely on the accuracy of any data, not use this application in any manner that violates your motor vehicle code or creates an unsafe driving condition and be bound by the terms and conditions listed on TruckBlox website at www.truckblox.com.  <p><br>User understands all objectional content will be subject to removal and termination of user account.  </p></div>'
                    });
                confirmPopup.then(function(res) {
                if(res) {
                    $scope.loading = $ionicLoading.show({
                        content: 'Sending',
                        animation: 'fade-in',
                        showBackdrop: true,
                        maxWidth: 200,
                        showDelay: 0
                    });
                    user.signUp(null, {
                        success: function(user) {
                            $ionicLoading.hide();
                            $rootScope.user = Parse.User.current();

                            window.localStorage.setItem("cachedUserName", $scope.user.username);
                            window.localStorage.setItem("cachedUP", $scope.user.password);

                            //NOTE: User object must be accessed through 'attributes'
                            console.log("Registered user: " + JSON.stringify(user));
                            
                            $state.go('tab.instructions', {
                                clear: true
                            });
                        },
                        error: function(user, error) {
                            $ionicLoading.hide();
                            if (error.code === 125) {
                                $scope.error.message = 'Please specify a valid email ' +
                                    'address';
                            } else if (error.code === 202) {
                                $scope.error.message = 'The email address or user name is already ' +
                                    'registered';
                            } else {
                                $scope.error.message = error.message;
                            }
                            $scope.$apply();
                        }
                    });
                } 
                else {
                    console.log('User canceled request to flag user');
                }
            });
            } else {
                $scope.error.message = 'Passwords do not match'
            }
        }
        catch (error) {
            console.log("Caught an error in Register ... " + error);
        }
    };

    $scope.goBackToLoginFromRegister = function() {
        console.log("Go back from Register was clicked");
        $state.go('welcome');
    }
})

//********** FORGOT PASSWORD **********//
.controller('ForgotPasswordController', function($scope, $state, $ionicLoading) {
    console.log('Inside ForgotPasswordController!');
    $scope.user = {};
    $scope.error = {};
    $scope.state = {
        success: false
    };

    $scope.reset = function() {
        $scope.loading = $ionicLoading.show({
            content: 'Sending',
            animation: 'fade-in',
            showBackdrop: true,
            maxWidth: 200,
            showDelay: 0
        });

        Parse.User.requestPasswordReset($scope.user.email, {
            success: function() {
                // TODO: show success
                $ionicLoading.hide();
                $scope.state.success = true;
                $scope.$apply();
                $state.go('welcome');
                alert('An email has been sent :)')
            },
            error: function(err) {
                $ionicLoading.hide();
                console.log(err.code);
                if (err.code === 125) {
                    $scope.error.message = 'Email address does not exist';
                } 
                else if (err.code === 205){
                    $scope.error.message = 'User not found. Please check and enter again.';
                } else if (err.code === 204){
                    $scope.error.message = 'Not an email address format';
                } 
                else {
                    $scope.error.message = 'An unknown error has occurred, ' + 'please try again';
                }
                $scope.$apply();
            }
        });
    };

    $scope.goBackToLogin = function() {
        $state.go('welcome');
    }
})

.controller('AppController', function($scope, $state, $rootScope, $ionicHistory, $stateParams) {
    console.log("Inside AppController");

    if ($stateParams.clear) {
        console.log("Clearing stateParams");
        $ionicHistory.clearHistory();
        $ionicHistory.clearCache();
    }
})

.controller('InstructionsCtrl', function($rootScope, $scope, $cordovaGeolocation, $ionicPlatform) {
    console.log("Inside InstructionsCtrl");
    var posOptions = null;

    // If android
    if($ionicPlatform.is('android')){
      console.log("Updating settings for Android (Geolocation)");
      posOptions = {timeout: 3600000, maximumAge:10000, enableHighAccuracy: true};
    }
    // If iPhone
    else {
      console.log("Updating settings for iOS (Geolocation)");
      posOptions = {timeout: 4000, maximumAge: 3000 , enableHighAccuracy: false};
    }

    var currentPositionID = $cordovaGeolocation.getCurrentPosition(posOptions);
        currentPositionID.then(
        null,
        function(err) {
            console.log("Error watching geoLocation, err=" + JSON.stringify(err));
        },
        function(position) {
            $rootScope.currentLiveLatitude  = position.coords.latitude;
            $rootScope.currentLiveLongitude = position.coords.longitude;
    });

	$scope.reportClick = function () {
		alert("Instructions Button pressed");
	};
});
